import logging
import shutil
import sys
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Union

from jinja2 import Environment, PackageLoader, select_autoescape
from slugify import slugify

from nprompter import CONTENT_DIRECTORY
from nprompter.notion_client import NotionClient

logger = logging.getLogger("NotionScripting")

env = Environment(loader=PackageLoader("nprompter"), autoescape=select_autoescape())
script_template = env.get_template("script.html")
index_template = env.get_template("index.html")


@dataclass
class Link:
    link: str
    name: str


class NotionProcessor:
    def __init__(self, notion_client: NotionClient, content_root: Union[str, None] = None):
        self.notion_client = notion_client
        self.links = []
        self.database_name = None
        self.folder = None
        self.content_root = content_root or CONTENT_DIRECTORY

    def create_scripts(self, database_id: str, section_name: str):
        database = self.notion_client.query_database(database_id)
        database_title = database["title"][0]["text"]["content"]

        self.start(database_title)

        pages = self.notion_client.get_pages(section_name, database_id)
        for page in pages:
            page_id = page["id"]
            blocks = self.notion_client.get_blocks(page_id)
            self.add_page(page_title=page["properties"]["Name"]["title"][0]["text"]["content"], blocks=blocks)
        self.finish(database_title)

    def start(self, database_name):
        self.folder = Path(self.content_root, slugify(database_name))
        try:
            shutil.rmtree(self.folder)
        except FileNotFoundError:
            pass
        except NotADirectoryError:
            print(f"The path {self.folder} exists but it is not a directory", file=sys.stderr)
            raise
        self.folder.mkdir(parents=True, exist_ok=True)

    def finish(self, database_title):
        index = index_template.render(elements=self.links, title=database_title)
        with open(Path(self.content_root, "index.html"), "w", encoding="utf8") as writable:
            writable.write(index)

    def process_paragraph_blog(self, block):
        return self.process_text_block_blog(block, "paragraph")

    def process_bulleted_list(self, block):
        return self.process_text_block_blog(block, "bulleted_list_item")

    def process_text_block_blog(self, block, node, make_paragraph=True):
        block_content = []
        for content in block[node]["text"]:
            if text_content := content.get("text"):
                text = text_content["content"]
                if content["annotations"].get("bold"):
                    text = f"<b>{text}</b>"
                if content["annotations"].get("italic"):
                    text = f"<i>{text}</i>"
                text = f"<span>{text}</span>"
            elif equation := content.get("equation"):
                text = f"\\({equation['expression']}\\)"
            else:
                text = None
                logger.warning(f"No action for {content['type']}")
            if text is not None:
                block_content.append(text)
        if make_paragraph:
            result = "<p>" + ("".join(block_content)) + "</p>"
        else:
            result = "".join(block_content)
        return result

    def add_page(self, page_title: str, blocks: List[Dict]) -> str:
        elements = []
        for block in blocks:
            block_type = block["type"]

            if block_type == "paragraph":
                result = self.process_paragraph_blog(block)
            elif block_type == "image":
                if "external" not in block["image"]:
                    image_url = block["image"]["file"]["url"]
                else:
                    image_url = block["image"]["external"]["url"]
                result = f'<img src="{image_url}" />'
            elif block_type == "bulleted_list_item":
                result = self.process_bulleted_list(block)
            elif block_type.startswith("heading_"):
                h_size = int(block_type[-1])
                content = self.process_text_block_blog(block, block_type, make_paragraph=False)
                result = f"<h{h_size}>{content}</h{h_size}>"
            elif block_type == "equation":
                result = f"$${block['equation']['expression']}$$"
            else:
                result = None
                logger.error(f"No action for block of type {block_type}")
            if result is not None:
                elements.append(result)

        content = script_template.render(elements=elements, title=page_title)

        file_name = Path(self.folder, Path(slugify(page_title) + ".html"))
        with open(file_name, "w", encoding="utf8") as writeable:
            writeable.write(content)

        parent_directory = str(file_name)[len(CONTENT_DIRECTORY) :]  # noqa

        self.links.append(Link(parent_directory, name=page_title))
        return content



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nprompter']

package_data = \
{'': ['*'], 'nprompter': ['templates/*', 'web/*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0',
 'python-dotenv>=0.19.0,<0.20.0',
 'python-slugify>=5.0.2,<6.0.0',
 'requests>=2.26.0,<3.0.0',
 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['nprompter = nprompter.__main__:app']}

setup_kwargs = {
    'name': 'nprompter',
    'version': '1.2.0',
    'description': '',
    'long_description': None,
    'author': 'Antonio Feregrino',
    'author_email': 'antonio.feregrino@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import http.server
import logging
import os
import shutil
import socketserver
import webbrowser
from importlib.resources import path
from typing import Any, Union

import typer
from dotenv import load_dotenv

from nprompter import CONTENT_DIRECTORY
from nprompter.notion_client import NotionClient
from nprompter.notion_processor import NotionProcessor

load_dotenv()
app = typer.Typer(add_completion=False)

notion_api_key = os.environ["NOTION_API_KEY"]
notion_version = "2021-05-13"

notion_client = NotionClient(notion_api_key, notion_version)

logger = logging.getLogger("NotionScripting")


@app.command()
def download(
    database_id: str = typer.Argument("", envvar="DATABASE_ID"),
    section_name: str = "Written",
    content_root: Union[str, None] = None,
):
    notion_processor = NotionProcessor(notion_client, content_root=content_root)
    notion_processor.create_scripts(database_id, section_name)


def copy_assets():
    with path(__package__, "web") as assets_folder:
        src_files = os.listdir(assets_folder)
        for file_name in src_files:
            full_file_name = os.path.join(assets_folder, file_name)
            if os.path.isfile(full_file_name):
                shutil.copy(full_file_name, CONTENT_DIRECTORY)


@app.command()
def serve(port: int = 8889):

    copy_assets()

    class CustomHandler(http.server.SimpleHTTPRequestHandler):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, directory=CONTENT_DIRECTORY, **kwargs)

        def log_message(self, format: str, *args: Any) -> None:
            pass

    socketserver.TCPServer.allow_reuse_address = True
    with socketserver.TCPServer(("", port), CustomHandler) as httpd:
        location = f"http://localhost:{port}"
        print(f"Serving at http://localhost:{port}")
        webbrowser.open(location)
        httpd.serve_forever()


if __name__ == "__main__":
    app()
